@@ -43,14 +43,20 @@ class NotesDB(utils.SubjectMixin):
         fnlist = glob.glob(self.helper_key_to_fname('*'))
         self.notes = {}
         for fn in fnlist:
-            n = json.load(open(fn, 'rb'))
-            # we always have a localkey, also when we don't have a note['key'] yet (no sync)
-            localkey = os.path.splitext(os.path.basename(fn))[0]
-            self.notes[localkey] = n
-            # we maintain in memory a timestamp of the last save
-            # these notes have just been read, so at this moment
-            # they're in sync with the disc.
-            n['savedate'] = now
+            try:
+                n = json.load(open(fn, 'rb'))
+
+            except ValueError, e:
+                logging.error('Error parsing %s: %s' % (fn, str(e)))
+
+            else:
+                # we always have a localkey, also when we don't have a note['key'] yet (no sync)
+                localkey = os.path.splitext(os.path.basename(fn))[0]
+                self.notes[localkey] = n
+                # we maintain in memory a timestamp of the last save
+                # these notes have just been read, so at this moment
+                # they're in sync with the disc.
+                n['savedate'] = now
         
         # initialise the simplenote instance we're going to use
         # this does not yet need network access
@@ -115,7 +121,10 @@ class NotesDB(utils.SubjectMixin):
 
         if search_string:
             try:
-                sspat = re.compile(search_string)
+                if self.config.case_sensitive == 0:
+                    sspat = re.compile(search_string, re.I)
+                else:
+                    sspat = re.compile(search_string)
             except re.error:
                 sspat = None
             
@@ -126,20 +135,42 @@ class NotesDB(utils.SubjectMixin):
         for k in self.notes:
             n = self.notes[k]
             c = n.get('content')
-            if not n.get('deleted') and (not sspat or sspat.search(c)):
-                # we have to store our local key also
-                filtered_notes.append(utils.KeyValueObject(key=k, note=n))
+            if self.config.search_tags == 1:
+                t = n.get('tags')
+                if not n.get('deleted') and sspat:
+                   if filter(sspat.search, t):
+                       # we have to store our local key also
+                       filtered_notes.append(utils.KeyValueObject(key=k, note=n, tagfound=1))
+                   elif sspat.search(c):
+                       # we have to store our local key also
+                       filtered_notes.append(utils.KeyValueObject(key=k, note=n, tagfound=0))
+                elif not n.get('deleted') and not sspat:
+                    # we have to store our local key also
+                    filtered_notes.append(utils.KeyValueObject(key=k, note=n, tagfound=0))
+            else:
+                if not n.get('deleted') and (not sspat or sspat.search(c)):
+                    # we have to store our local key also
+                    filtered_notes.append(utils.KeyValueObject(key=k, note=n, tagfound=0))
             
         if self.config.sort_mode == 0:
-            # sort alphabetically on title
-            filtered_notes.sort(key=lambda o: utils.get_note_title(o.note))
+            if self.config.pinned_ontop == 0:
+                # sort alphabetically on title
+                filtered_notes.sort(key=lambda o: utils.get_note_title(o.note))
+            else:
+                filtered_notes.sort(utils.sort_by_title_pinned)
             
         else:
-            # last modified on top
-            filtered_notes.sort(key=lambda o: -float(o.note.get('modifydate', 0)))
+            if self.config.pinned_ontop == 0:
+                # last modified on top
+                filtered_notes.sort(key=lambda o: -float(o.note.get('modifydate', 0)))
+            else:
+                filtered_notes.sort(utils.sort_by_modify_date_pinned, reverse=True)
             
         return filtered_notes
-    
+
+    def get_note(self, key):
+        return self.notes[key]
+
     def get_note_content(self, key):
         return self.notes[key].get('content')
     
@@ -407,7 +438,7 @@ class NotesDB(utils.SubjectMixin):
                     self.notify_observers('progress:sync_full', utils.KeyValueObject(msg='Synced modified note %d to server.' % (ni,)))
                         
                 else:
-                    raise SyncError("Sync step 1 error: Could not update note to server.")
+                    raise SyncError("Sync step 1 error - Could not update note to server")
              
         # 2. if remote syncnum > local syncnum, update our note; if key is new, add note to local.
         # this gets the FULL note list, even if multiple gets are required
@@ -422,6 +453,7 @@ class NotesDB(utils.SubjectMixin):
         
         server_keys = {}
         lennl = len(nl)
+        sync_from_server_errors = 0
         for ni,n in enumerate(nl):
             k = n.get('key')
             server_keys[k] = True
@@ -436,19 +468,28 @@ class NotesDB(utils.SubjectMixin):
                     if ret[1] == 0:
                         self.notes[k].update(ret[0])
                         local_updates[k] = True
+                        # in both cases, new or newer note, syncdate is now.
+                        self.notes[k]['syncdate'] = now
                         self.notify_observers('progress:sync_full', utils.KeyValueObject(msg='Synced newer note %d (%d) from server.' % (ni,lennl)))
-                        
+
+                    else:
+                        logging.error('Error syncing newer note %s from server: %s' % (k, ret[0]))
+                        sync_from_server_errors+=1
+
             else:
                 # new note
                 ret = self.simplenote.get_note(k)
                 if ret[1] == 0:
                     self.notes[k] = ret[0]
                     local_updates[k] = True
+                    # in both cases, new or newer note, syncdate is now.
+                    self.notes[k]['syncdate'] = now
                     self.notify_observers('progress:sync_full', utils.KeyValueObject(msg='Synced new note %d (%d) from server.' % (ni,lennl)))
-                    
-            # in both cases, new or newer note, syncdate is now.
-            self.notes[k]['syncdate'] = now
-                    
+
+                else:
+                    logging.error('Error syncing new note %s from server: %s' % (k, ret[0]))
+                    sync_from_server_errors+=1
+
         # 3. for each local note not in server index, remove.     
         for lk in self.notes.keys():
             if lk not in server_keys:
@@ -465,6 +506,8 @@ class NotesDB(utils.SubjectMixin):
                 os.unlink(fn)
 
         self.notify_observers('progress:sync_full', utils.KeyValueObject(msg='Full sync complete.'))
+
+        return sync_from_server_errors
         
     def set_note_content(self, key, content):
         n = self.notes[key]
@@ -474,6 +517,35 @@ class NotesDB(utils.SubjectMixin):
             n['modifydate'] = time.time()
             self.notify_observers('change:note-status', utils.KeyValueObject(what='modifydate', key=key))
 
+    def set_note_tags(self, key, tags):
+        n = self.notes[key]
+        old_tags = n.get('tags')
+        tags = utils.sanitise_tags(tags)
+        if tags != old_tags:
+            n['tags'] = tags
+            n['modifydate'] = time.time()
+            self.notify_observers('change:note-status', utils.KeyValueObject(what='modifydate', key=key))
+
+    def set_note_pinned(self, key, pinned):
+        n = self.notes[key]
+        old_pinned = utils.note_pinned(n)
+        if pinned != old_pinned:
+            if 'systemtags' not in n:
+                n['systemtags'] = []
+
+            systemtags = n['systemtags']
+
+            if pinned:
+                # which by definition means that it was NOT pinned
+                systemtags.append('pinned')
+
+            else:
+                systemtags.remove('pinned')
+
+            n['modifydate'] = time.time()
+            self.notify_observers('change:note-status', utils.KeyValueObject(what='modifydate', key=key))
+
+
     def worker_save(self):
         while True:
             o = self.q_save.get()
