@@ -529,7 +529,17 @@ class View(utils.SubjectMixin):
 
     def get_continuous_rendering(self):
         return self.continuous_rendering.get()
-        
+
+    def get_selected_text(self):
+        """
+        Return note text that has been selected by user.
+        """
+
+        try:
+            return self.text_note.selection_get()
+        except tk.TclError:
+            return ''
+
     def get_text(self):
         # err, you have to specify 1.0 to END, and NOT 0 to END like I thought.
         # also, see the comment by Bryan Oakley to
@@ -732,9 +742,17 @@ class View(utils.SubjectMixin):
         edit_menu.add_command(label="Find", accelerator="Ctrl+F",
                               underline=0, command=lambda: self.search_entry.focus())
         self.root.bind_all("<Control-f>", lambda e: self.search_entry.focus())
-        
-        
 
+        # TOOLS ########################################################
+        tools_menu = tk.Menu(menu, tearoff=False)
+        menu.add_cascade(label="Tools", underline=0, menu=tools_menu)
+
+        tools_menu.add_command(label="Word Count",
+            underline=0, command=self.word_count)
+
+        # the internet thinks that multiple modifiers should work, but this didn't
+        # want to.
+        #self.root.bind_all("<Control-Shift-c>", lambda e: self.word_count())
 
         # HELP ##########################################################
         help_menu = tk.Menu(menu, tearoff=False)
@@ -1106,7 +1124,24 @@ class View(utils.SubjectMixin):
         # single keystroke.
         self.activate_links()
         self.activate_search_string_highlights()
-                
+
+    def is_note_different(self, note):
+        """
+        Determine if note would cause a UI update.
+        """
+
+        if self.get_text() != note.get('content'):
+            return True
+
+        tags = note.get('tags', [])
+        # get list of string tags from ui
+        ui_tags = utils.sanitise_tags(self.tags_entry_var.get())
+        if ui_tags != tags:
+            return True
+
+        if bool(self.pinned_checkbutton_var.get()) != bool(utils.note_pinned(note)):
+            return True
+
     def observer_notes_list(self, notes_list_model, evt_type, evt):
         if evt_type == 'set:list':
             # re-render!
@@ -1123,7 +1158,7 @@ class View(utils.SubjectMixin):
     def set_status_text(self, txt):
         self.statusbar.set_status(txt)
         
-    def set_note_data(self, note, reset_undo=True):
+    def set_note_data(self, note, reset_undo=True, content_unchanged=False):
         """Replace text in editor with content.
         
         This is usually called when a new note is selected (case 1), or
@@ -1131,12 +1166,17 @@ class View(utils.SubjectMixin):
         
         @param reset_undo: Set to False if you don't want to have the undo
         buffer to reset.
+        @param content_unchanged: Set to True if you know that the content
+        has not changed, only the tags and pinned status.
         """
-        
-        self.text_note.delete(1.0, tk.END) # clear all
+
+        if not content_unchanged:
+            self.text_note.delete(1.0, tk.END) # clear all
 
         if note is not None:
-            self.text_note.insert(tk.END, note['content'])
+            if not content_unchanged:
+                self.text_note.insert(tk.END, note['content'])
+
             # default to an empty array for tags
             tags=note.get('tags', [])
             self.tags_entry_var.set(','.join(tags))
@@ -1184,17 +1224,55 @@ class View(utils.SubjectMixin):
         """
         Update currently selected note's data.
 
-        This is called only by the event handler for the per-note on-demand
-        syncing.
+        This is called when the user triggers a per-note sync and a newer
+        note comes back, but also when the search string changes, and the
+        currently selected note gets a newer version due to background or
+        foreground syncing.
+
+        We take care only to update the note content if it has actually
+        changed, to minimise visual glitches.
         """
 
-        # store cursor position
-        cursor_pos = self.text_note.index(tk.INSERT)
         # the user is not changing anything, so we don't want the event to fire
         self.mute_note_data_changes()
-        self.set_note_data(note)
-        self.text_note.mark_set(tk.INSERT, cursor_pos)
+
+        current_content = self.get_text()
+        new_content = note.get('content', '')
+
+        if new_content != current_content:
+            # store cursor position
+            cursor_pos = self.text_note.index(tk.INSERT)
+            # also store visible window
+            first, last = self.text_note.yview()
+
+            # set new note contents, pinned status and tags
+            # but keep user's undo buffer
+            self.set_note_data(note, reset_undo=False)
+
+            # restore visible window
+            self.text_note.yview('moveto', first)
+            self.text_note.mark_set(tk.INSERT, cursor_pos)
+            self.activate_links()
+            self.activate_search_string_highlights()
+
+        else:
+            # we know the content is the same, so we only set the rest
+            # obviously keep user's undo buffer.
+            self.set_note_data(note, reset_undo=False, content_unchanged=True)
+
+        # reactivate event handlers
         self.unmute_note_data_changes()
 
 
-        
+    def word_count(self):
+        """
+        Display count of total words and selected words in a dialog box.
+        """
+
+        sel = self.get_selected_text()
+        slen = len(sel.split())
+
+        txt = self.get_text()
+        tlen = len(txt.split())
+
+        self.show_info('Word Count', '%d words in total\n%d words in selection' % (tlen,slen))
